package org.apache.bcel.verifier;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

public class VerifierFactory {
   private static final Map<String, Verifier> MAP = new HashMap();
   private static final List<VerifierFactoryObserver> OBSVERVERS = new Vector();

   public static void attach(VerifierFactoryObserver o) {
      OBSVERVERS.add(o);
   }

   static void clear() {
      MAP.clear();
      OBSVERVERS.clear();
   }

   public static void detach(VerifierFactoryObserver o) {
      OBSVERVERS.remove(o);
   }

   public static Verifier getVerifier(String fullyQualifiedClassName) {
      return (Verifier)MAP.computeIfAbsent(fullyQualifiedClassName, (k) -> {
         Verifier v = new Verifier(k);
         notify(k);
         return v;
      });
   }

   public static Verifier[] getVerifiers() {
      return (Verifier[])MAP.values().toArray(Verifier.EMPTY_ARRAY);
   }

   private static void notify(String fullyQualifiedClassName) {
      OBSVERVERS.forEach((vfo) -> {
         vfo.update(fullyQualifiedClassName);
      });
   }

   private VerifierFactory() {
   }
}
